#ifndef GGJ_2021_CLIENT_HPP
#define GGJ_2021_CLIENT_HPP

#include <memory>
#include <queue>
#include "network/ISocket.hpp"

#include <cstdint>

struct Client {

    static std::queue<uint16_t> freeId;

    explicit Client(std::shared_ptr<network::ISocket> &socket) : socket(socket), port(0), isReading(false), connected(true)
    {
        static uint16_t id_ = 0;

        if (Client::freeId.size() != 0) {
            id = Client::freeId.front();
            Client::freeId.pop();
        } else {
            id = id_++;
        }
    }
    Client(const Client &) = delete;
    Client &operator=(const Client &) = delete;

    ~Client()
    {
        Client::freeId.push(id);
    };

    std::shared_ptr<network::ISocket> socket;
    std::queue<std::string> data;
    int port;
    bool isReading;
    int16_t idRoom = -1;
    uint16_t id;
    bool connected;
};

#endif //GGJ_2021_CLIENT_HPP
