//
// Created by mockie on 30/01/2021.
//

#ifndef GGJ_2021_SERVERMANAGER_HPP
#define GGJ_2021_SERVERMANAGER_HPP

#include <boost/asio/io_context.hpp>
#include <memory>
#include <vector>
#include <map>
#include <string>
#include <thread>
#include <functional>

#include "network/IServer.hpp"
#include "network/Protocol.hpp"

#include "server/Client.hpp"
#include "server/IGame.hpp"
#include "network/UDPSocket.hpp"
using namespace network;

class ServerManager {
public:
    ServerManager(int argc, char **argv);

    ~ServerManager() = default;
    ServerManager(const ServerManager &) = delete;
    ServerManager &operator=(const ServerManager &) = delete;

    void initialize();
    void run();
    void close(){};

private:
    bool handleAccept(std::shared_ptr<network::ISocket> &connection);
    void writeHandler(Client &clientInfo);
    int readHandler(Client &clientInfo, const std::string &str);
    void disconnectHandler(Client &clientInfo);

    static std::string prepareHeader(unsigned short code, unsigned int size);

    void startGame(Client &client, const std::string &message);
    void unknownRequest(Client &client, const std::string &message);

    std::vector<std::shared_ptr<Client>> _clients;
    std::vector<std::unique_ptr<IGame>> _games;
    std::unique_ptr<network::IServer> _server;
    std::shared_ptr<network::ISocket> _socketUDP;
    boost::asio::io_context _context;
    int _port;

    static const std::map<unsigned short, void (ServerManager::*)(Client &client, const std::string &message)> eventsHandlers;
};

#endif //GGJ_2021_SERVERMANAGER_HPP