/*
** EPITECH PROJECT, 2020
** RType
** File description:
** FileParserManager
*/

#ifndef R_TYPE_SERVER_FILEPARSERMANAGER_HPP
#define R_TYPE_SERVER_FILEPARSERMANAGER_HPP

#include <fstream>
#include <iostream>
#include <string>
#include <regex>
#include <vector>
#include <algorithm>
#include <filesystem>
#if _WIN32
#include <windows.h>
#else
#include <sys/stat.h>
#endif

namespace fileParser {
    struct Frame {
        std::vector<int> rect;
        double duration;
    };
    struct AnimData {
        std::string animTitle;
        uint16_t animId;
        bool loops;
        std::string filepath;
        std::vector<Frame> frames;
    };
    struct DisplayableEntityInfo {
        std::string resourcePath;
        std::vector<AnimData> animations;
    };

    class FileParserManager {
    public:
        FileParserManager() = default;
        ~FileParserManager();
        FileParserManager(const FileParserManager &) = default;
        FileParserManager &operator=(const FileParserManager &) = default;

        static void replaceSeparators(std::string&);
        bool isOpen(const std::string &path);
        void read(const std::string& path);
        std::vector<DisplayableEntityInfo> loadConfigFiles(const std::string &path);
        DisplayableEntityInfo getDisplayableEntityInfo(const std::string &path);
    private:
        std::vector<std::filesystem::path> retrieveConfigFileList(const std::string &path);
        bool loadAnimations(DisplayableEntityInfo &spriteInfo);
        bool configFileHasAnimations();
        std::string isValidFileSpriteConfig();
        bool retrieveAnimation(const std::string&, DisplayableEntityInfo &spriteInfo, bool loops, uint16_t &animId);
        std::ifstream _currentFile;
        std::string _currentPath;
        bool _isOpen;
        static uint16_t animId;
    };
}

#endif //R_TYPE_SERVER_FILEPARSERMANAGER_HPP
