/*
** EPITECH PROJECT, 2020
** RType
** File description:
** FileParserManager
*/

#include "fileParser/FileParserManager.hpp"

uint16_t fileParser::FileParserManager::animId = 0;

void fileParser::FileParserManager::replaceSeparators(std::string &toReplace) {
#if _WIN32
    std::string separatorToFind = "/";
    std::string separatorToReplace = "\\";
#else
    std::string separatorToFind = "\\";
    std::string separatorToReplace = "/";
#endif
    size_t pos = toReplace.find(separatorToFind);
    while( pos != std::string::npos)
    {
        toReplace.replace(pos, separatorToFind.size(), separatorToReplace);
        pos = toReplace.find(separatorToFind, pos + separatorToReplace.size());
    }
}

bool fileParser::FileParserManager::isOpen(const std::string& path) {
    std::string filepath = path;
    replaceSeparators(filepath);

    _currentFile.open(filepath);
    return _currentFile.is_open();
}

void fileParser::FileParserManager::read(const std::string& path) {
    if (isOpen(path)) {
        std::string line;
        while (getline(_currentFile, line)) {
            std::cout << line << std::endl;
        }
        _currentFile.close();
    }
}

fileParser::DisplayableEntityInfo fileParser::FileParserManager::getDisplayableEntityInfo(const std::string &path) {
    size_t linePos;
    DisplayableEntityInfo info = {"ERROR", std::vector<AnimData>()};
    if (isOpen(path) && !(info.resourcePath = isValidFileSpriteConfig()).empty()) {
        if ((linePos = configFileHasAnimations()) != 0) {
            loadAnimations(info);
        }
        _currentFile.close();
    }
    return (info);
}

std::string fileParser::FileParserManager::isValidFileSpriteConfig() {
    size_t pos;
    std::string line, resourcePath;
    std::string identifier = "filepath=";
    bool filePathFound = false;
    while (getline(_currentFile, line)) {
        if ((pos = line.find(identifier) != std::string::npos) && (pos + identifier.length() != std::string::npos)) {
            filePathFound = true;
            resourcePath = ASSETS_PATH + line.substr(line.find('=') + 1);
            replaceSeparators(resourcePath);
        }
        if (line.find("type=texture") != std::string::npos) {
            return (filePathFound ? resourcePath : "");
        }
    }
    return "";
}

bool fileParser::FileParserManager::configFileHasAnimations() {
    std::string line;
    std::string identifier = "animations=";
    while (getline(_currentFile, line)) {
        if (line.find(identifier) != std::string::npos) {
            return true;
        }
    }
    return false;
}

bool fileParser::FileParserManager::loadAnimations(DisplayableEntityInfo &info) {
    std::string line;
    std::string loopAnimIdentifier = "=L";
    std::string straightAnimIdentifier = "=S";
    bool parseErr = false;

    while (getline(_currentFile, line)) {
        if (line.length() > 2) {
            if (line.find(loopAnimIdentifier) != std::string::npos) {
                parseErr = !retrieveAnimation(line.substr(0, line.length() - 2), info, true, animId);
            } else if (line.find(straightAnimIdentifier) != std::string::npos) {
                parseErr = !retrieveAnimation(line.substr(0, line.length() - 2), info, false, animId);
            }
        }
    }
    return parseErr;
}

bool fileParser::FileParserManager::retrieveAnimation(const std::string& animationName, DisplayableEntityInfo &info, bool loops, uint16_t &animId) {
    std::string line;
    size_t pos;
    size_t linePos = _currentFile.tellg();
    size_t frameCount = 0;
    bool animExistsYet = false;
    std::smatch matches;

    while (getline(_currentFile, line)) {
        pos = line.find(animationName + "=");
        if (pos != std::string::npos) {
            line.erase(pos, animationName.length() + 1);
            if(std::regex_match(line, matches, std::regex("(\\d+):(\\d+):(\\d+):(\\d+)\\|(\\d+.\\d+)$"))) {
                if (!animExistsYet)
                    info.animations.emplace_back(AnimData{animationName, animId, loops, info.resourcePath, std::vector<Frame>()});
                animExistsYet = true;
                info.animations.back().frames.emplace_back(Frame{});
                info.animations.back().frames[frameCount].rect.emplace_back(std::stoi(matches[1]));
                info.animations.back().frames[frameCount].rect.emplace_back(std::stoi(matches[2]));
                info.animations.back().frames[frameCount].rect.emplace_back(std::stoi(matches[3]));
                info.animations.back().frames[frameCount].rect.emplace_back(std::stoi(matches[4]));
                info.animations.back().frames[frameCount].duration = std::stof(matches[5]);
                frameCount += 1;
            } else {
                return false;
            }
        } else {
            _currentFile.seekg(linePos);
            break;
        }
        linePos = _currentFile.tellg();
    }
    animId = animId + 1;
    return true;
}

std::vector<std::filesystem::path> fileParser::FileParserManager::retrieveConfigFileList(const std::string &path) {
    std::filesystem::path directory(path);
    std::vector<std::filesystem::path> paths;

    if (std::filesystem::exists(directory) && std::filesystem::is_directory(directory)) {
        copy(std::filesystem::directory_iterator(directory), std::filesystem::directory_iterator(), back_inserter(paths));
        sort(paths.begin(), paths.end());
    }
    return paths;
}

std::vector<fileParser::DisplayableEntityInfo> fileParser::FileParserManager::loadConfigFiles(const std::string &path) {
    std::vector<std::filesystem::path> paths;
    std::vector<fileParser::DisplayableEntityInfo> entities;
    fileParser::DisplayableEntityInfo entity;
    if (!(paths = retrieveConfigFileList(path)).empty()) {
        for(auto it = std::begin(paths); it != std::end(paths); ++it) {
            if (std::regex_match(it->filename().string(),std::regex("^[a-zA-Z]+\\.config$"))) {
                entities.emplace_back(getDisplayableEntityInfo(it->string()));
            }
        }
    }
    return entities;
}

fileParser::FileParserManager::~FileParserManager()
{
    animId = 0;
}
