/*
** EPITECH PROJECT, 2020
** RType
** File description:
** GraphicalHandler
*/

#ifndef _GRAPHICAL_ENGINE_HPP_
#define _GRAPHICAL_ENGINE_HPP_

#include "graphicalEngine/FontManager.hpp"
#include "graphicalEngine/Window.hpp"
#include "graphicalEngine/KeyboardManager.hpp"
#include "graphicalEngine/SpriteManager.hpp"
#include "graphicalEngine/ShapeHandler.hpp"

namespace graphicalEngine {
    class GraphicalHandler {
        public:
            GraphicalHandler(unsigned int windowWidth, unsigned int windowHeight, const std::string& windowTitle):
                _renderWindow(std::make_shared<Window>(windowWidth, windowHeight, windowTitle)),
                _kbm(std::weak_ptr<Window>(this->_renderWindow))
            {}
            GraphicalHandler(const GraphicalHandler &) = delete;
            GraphicalHandler &operator=(const GraphicalHandler &) = delete;

            ~GraphicalHandler() = default;

            void display();
            bool isOpen() const {
                return _renderWindow->isOpen();
            }

            void draw(const sf::Drawable &drawable) {
                _renderWindow->draw(drawable);
            }

            void draw(const Sprite &sprite) {
                _renderWindow->draw(sprite);
            }

            void clearAll();

            sf::Vector2i getMousePos() const;
            bool isMouseClicked(sf::Mouse::Button button = sf::Mouse::Left) const;

        public:
            std::shared_ptr<Window> _renderWindow;
            SpriteManager _sm;
            KeyboardManager _kbm;
            sf::Mouse _mouse;
            FontManager _fontManager;
    };
}

#endif //_GRAPHICALENGINE_HPP_
