#ifndef _GRAPHICAL_SPRITE_HPP_
#define _GRAPHICAL_SPRITE_HPP_

#include <SFML/Graphics/Sprite.hpp>
#include <memory>
#include <map>
#include "fileParser/FileParserManager.hpp"
#include "graphicalEngine/TextureData.hpp"
#include "graphicalEngine/Animation.hpp"

namespace graphicalEngine {

    class Sprite {
        public:
            Sprite() = default;
            Sprite(const Sprite &) = default;
            Sprite &operator=(const Sprite &) = default;

            explicit Sprite(std::reference_wrapper<TextureData> textureData) :
                _sprite(textureData.get().getTextureData())
            {}

            Sprite(std::reference_wrapper<TextureData> textureData, sf::Vector2f position) :
                _sprite(textureData.get().getTextureData())
            {
                _sprite.setPosition(position);
            }

            void setTexture(std::reference_wrapper<TextureData> textureData) {
                _sprite.setTexture(textureData.get().getTextureData());
            }

            const sf::Sprite &getSprite() const {
                return _sprite;
            }

            sf::Vector2f getPosition() {
                return _sprite.getPosition();
            }

            void setSize(float w, float h) {
                auto bounds = _sprite.getGlobalBounds();
                _sprite.scale(w / bounds.width, h / bounds.height);
            }

            void setPosition(float x, float y) {
                sf::Vector2f position(x, y);
                this->_sprite.setPosition(position);
            }

            void setAnimation(const fileParser::AnimData &animation) {
                _animation = Animation();
                for (auto itFrame = std::begin(animation.frames); itFrame != std::end(animation.frames); ++itFrame) {
                    _animation.addFrame({{itFrame->rect[0], itFrame->rect[1],itFrame->rect[2],itFrame->rect[3]}, itFrame->duration});
                    _animation.loop(animation.loops);
                }
                _animation.update(0, _sprite);
            }

            Animation &getAnimation() {
                return _animation;
            }

            void pauseCurrentAnimation(bool pause) {
                _animation.pause(pause);
            }

            void updateCurrentAnimation(double timeElapsed) {
                _animation.update(timeElapsed, _sprite);
            }

            ~Sprite() = default;

        private:
            sf::Sprite _sprite;
            Animation _animation;
    };
}

#endif //_GRAPHICAL_SPRITE_HPP_
