#ifndef _GRAPHICAL_SPRITEMANAGER_HPP_
#define _GRAPHICAL_SPRITEMANAGER_HPP_

#include <map>
#include <SFML/System/Clock.hpp>
#include "fileParser/FileParserManager.hpp"
#include "graphicalEngine/Sprite.hpp"

namespace graphicalEngine {
    class SpriteManager {
        public:
            SpriteManager() = default;
            ~SpriteManager() = default;
            SpriteManager(const SpriteManager &) = default;
            SpriteManager &operator=(const SpriteManager &) = default;

            size_t addSprite(const fileParser::AnimData &animation);
            void addTexture(const std::string &configurationFile);

            void moveSprite(size_t spriteId, float x, float y);

            Sprite &getSprite(size_t spriteId);
            const Sprite &getSprite(size_t spriteId) const;
            float getGlobalClockElapsedTime();
            float restartGlobalClock();

            void animateSprites();

            void clear();
            void clear(size_t spriteId);

        private:
            size_t _spriteCount = 0;
            std::map<size_t, Sprite> _loadedSprites;
            std::map<std::string, TextureData> _loadedTextures;
            sf::Clock _animClock;
            sf::Clock _globalClock;
    };
}

#endif //_GRAPHICAL_SPRITEMANAGER_HPP_
