#ifndef _GRAPHICAL_RENDER_WINDOW_HPP_
#define _GRAPHICAL_RENDER_WINDOW_HPP_

#include <SFML/Graphics/RenderWindow.hpp>
#include <SFML/Window/Event.hpp>
#include <iostream>
#include <map>
#include "Sprite.hpp"
#include "graphicalEngine/FontManager.hpp"
#include "graphicalEngine/ShapeHandler.hpp"
#include "graphicalEngine/SpriteManager.hpp"

namespace graphicalEngine {
    class Window {
        public:
            Window(unsigned int width, unsigned int height, const std::string &title) :
                _window(sf::VideoMode(width, height), title, sf::Style::Close /* | sf::Style::Fullscreen */),
                _title(title)
            {}
            Window(const Window &) = delete;
            Window &operator=(const Window) = delete;

            void display();
            void eventLoop();

            sf::VideoMode getRenderWindowSize() const;
            void setRenderWindowSize(unsigned int windowWidth, unsigned int windowHeight);

            std::string getTitle() const;
            void setRenderWindowTitle(const std::string &title);

            sf::RenderWindow &getWindowInstance() {
                return this->_window;
            }

            bool isOpen() const {
                return _window.isOpen();
            }

            uint32_t getTextEntered() const {return _textEntered;}

            void draw(const sf::Drawable &drawable);
            void draw(const Sprite &sprite);

        private:
            sf::RenderWindow _window;
            std::string _title;
            uint32_t _textEntered;
    };
}

#endif //_GRAPHICAL_RENDER_WINDOW_HPP_
