#include <iostream>
#include "graphicalEngine/Animation.hpp"

graphicalEngine::Animation::Animation() {
        _progress = _totalLength = 0.0;
}

void graphicalEngine::Animation::addFrame(SpriteFrame&& frame) {
    _totalLength += frame.duration;
    _frames.push_back(std::move(frame));
}

void graphicalEngine::Animation::update(double elapsed, sf::Sprite &target) {
    if (!_pause) {
        if (_progress < _totalLength)
            _progress += elapsed;
        double p = _progress;
        for (size_t i = 0; i < _frames.size(); i++) {
            p -= _frames[i].duration;

            if (p <= 0.0 || &(_frames[i]) == &_frames.back()) {
                target.setTextureRect(_frames[i].rect);
                break;
            }
        }
        if (_loop)
            while (_progress >= _totalLength)
                _progress -= _totalLength;
    } else {
        target.setTextureRect(_frames[_frame].rect);
    }
}

void graphicalEngine::Animation::pause(bool pause) {
    _pause = pause;
}

void graphicalEngine::Animation::loop(bool loop) {
    _loop = loop;
}

void graphicalEngine::Animation::setDisplayedFrame(size_t frame)
{
    _frame = frame;
}
