/*
** EPITECH PROJECT, 2020
** RType
** File description:
** SpriteManager
*/

#include "graphicalEngine/SpriteManager.hpp"

using namespace graphicalEngine;

size_t SpriteManager::addSprite(const fileParser::AnimData &animation) {
    auto it = _loadedTextures.find(animation.filepath);
    if (it == _loadedTextures.end()) {
        addTexture(animation.filepath);
        it = _loadedTextures.find(animation.filepath);
    }
    _loadedSprites.emplace(++_spriteCount, Sprite(it->second));
    _loadedSprites[_spriteCount].setAnimation(animation);
    return (_spriteCount);
}

void SpriteManager::addTexture(const std::string &configurationFile) {
    _loadedTextures.emplace(configurationFile, TextureData(configurationFile));
}

void SpriteManager::moveSprite(size_t spriteId, float x, float y) {
    auto it = _loadedSprites.find(spriteId);
    if (it != _loadedSprites.end())  {
        it->second.setPosition(it->second.getPosition().x + x, it->second.getPosition().y + y);
    }
}

Sprite &SpriteManager::getSprite(size_t spriteId)
{
    return _loadedSprites.at(spriteId);
}

const Sprite &SpriteManager::getSprite(size_t spriteId) const
{
    return _loadedSprites.at(spriteId);
}

void SpriteManager::clear()
{
    _loadedSprites.clear();
    _spriteCount = 0;
}

void SpriteManager::clear(size_t spriteid)
{
    _loadedSprites.erase(spriteid);
}

void SpriteManager::animateSprites() {
    for (auto sprite = std::begin(_loadedSprites); sprite != std::end(_loadedSprites); ++sprite) {
        sprite->second.updateCurrentAnimation(this->_animClock.getElapsedTime().asSeconds());
    }
    _animClock.restart();
}

float SpriteManager::getGlobalClockElapsedTime() {
    return (this->_globalClock.getElapsedTime().asSeconds());
}

float SpriteManager::restartGlobalClock() {
    return (this->_globalClock.restart().asSeconds());
}
