
/*
** EPITECH PROJECT, 2020
** B-CPP-500-NAN-5-1-R-Type
** File description:
** Protocol
*/

#ifndef PROTOCOL_HPP_
#define PROTOCOL_HPP_
#ifdef _WIN32
#else
#include <bits/stdint-uintn.h>
#endif
#define MAX_INPUT_SIZE 50
#define MAX_MESSAGE_SIZE 512

#ifdef __GNUC__
#define PACK(__Declaration__) __Declaration__ __attribute__((__packed__))
#endif

#ifdef _MSC_VER
#define PACK(__Declaration__) __pragma( pack(push, 1) ) __Declaration__ __pragma( pack(pop))
#endif

namespace network
{
    PACK(struct TCPHeader
    {
        unsigned short nb;
        unsigned int messageSize;
    });

    PACK(struct UDPEventClient
    {
        short idRoom;
        short idClient;
        char code;
        short messageSize;
    });

    PACK(struct UDPEventServer
    {
        char code;
        short messageSize;
        uint16_t nbImages;
        uint16_t nbMusics;
        uint16_t nbScores;
        uint16_t nbLifesPlayer;
        uint16_t nbLifesMonster;
    });

    PACK(struct UDPEntityImage
    {
        uint16_t resource;
        uint32_t id;
        int16_t x;
        int16_t y;
        uint16_t w;
        uint16_t h;
    });

    PACK(struct UDPEntityMusic
    {
        uint16_t resource;
    });

    PACK(struct UDPEntityScore
    {
        uint16_t player;
        int score;
    });

    PACK(struct UDPEntityLifePlayer
     {
         uint16_t player;
         int life;
     });

    PACK(struct TCPEntityScore
    {
        uint16_t player;
        int score;
    });

    PACK(struct UDPEntityLifeMonster
     {
         uint16_t player;
         int life;
    });

    namespace udpEntityType {
        enum UdpEntityType {
            ENTITY,
            MUSIC,
            SCORE,
            LIFE
        };
    }

    namespace response
    {
        enum Response
        {
            OK = 200,
            ROOM_CREATE = 300,
            JOIN_ROOM = 301,
            LEAVE_ROOM = 302,
            GAME_STARTING = 303,
            BAD_REQUEST = 400,
            UNAUTHORIZED = 401,
            NOT_FOUND = 404,
            INTERNAL_ERROR = 500,
            SERVICE_UNAVAILABLE = 503
        };
    }

    namespace request
    {
        enum Request
        {
            CREATE_ROOM = 1,
            JOIN_ROOM,
            LEAVE_ROOM,
            START_GAME
        };
    }

    namespace broadcast
    {
        enum Broadcast
        {
            PLAYER_JOIN_ROOM = 100,
            PLAYER_LEAVE_ROOM,
            GAME_START,
            GAME_END
        };
    } // namespace broadcast

    namespace event
    {
        enum Event
        {
            KEY_UP = 1,
            KEY_DOWN = 2,
            KEY_LEFT = 4,
            KEY_RIGHT = 8,
            KEY_SPACE = 16,
            KEY_X = 32,
            KEY_ESCAPE = 64,
        };

        enum Server
        {
            GAME_STATE = 1,
        };
    }

} // namespace babel::network


#endif /* !PROTOCOL_HPP_ */
