/*
** EPITECH PROJECT, 2020
** R-Type
** File description:
** UDPSocket
*/

#ifndef __UDPSOCKET_H_
#define __UDPSOCKET_H_

#include "network/ISocket.hpp"

namespace network {
    class UDPSocket: public ISocket {
        public:
            UDPSocket() = delete;
            ~UDPSocket() override = default;

            UDPSocket(const UDPSocket &) = delete;
            UDPSocket &operator=(const UDPSocket &) = delete;

            static std::shared_ptr<UDPSocket> create(boost::asio::io_context &ioContext, unsigned short port);
            boost::asio::ip::udp::socket &socket();

            void write(const std::string &toAdd) override;
            void write(const std::string &toAdd, int size) override;
            void write_to(const std::string &toAdd, const std::string &ip, unsigned short port);
            void setWriteFct(const std::function<void ()> &handler) override;

            void setReadFct(const std::function<int(const std::string &data)> &handler) override;
            void read() override;

            void setDisconnectFct(const std::function<void()> &handler) override;

            void asyncConnect(
                const std::string &ip,
                unsigned short port,
                const std::function<void(const boost::system::error_code& ec)> &handler
            ) override;
            void asyncConnect(
                const std::array<unsigned char, 4> &ip,
                unsigned short port,
                const std::function<void(const boost::system::error_code& ec)> &handler
            ) override;

            [[nodiscard]] bool isConnected() const override;

            [[nodiscard]] std::array<unsigned char, 4> getIp() const override;

        private:
            UDPSocket(boost::asio::io_context &ioContext, unsigned short port);

            void handleDisconnect();

            void readHandler(const boost::system::error_code &error, size_t size);

        private:

            std::function<void ()> _disconnectFct = nullptr;
            std::function<int(const std::string &data)> _readFct = nullptr;
            std::function<void()> _writeFct = nullptr;

            boost::asio::ip::udp::socket _socket;

            bool _connected = true;

            char _bufferRead[10000];
            std::string _savedBuffer;

    };
}

#endif // __UDPSOCKET_H_
